package fr.natan.cleanarchitecturecompanyservice.infrastructure.ports.input.controller;

import fr.natan.cleanarchitecturecompanyservice.domain.entity.Company;
import fr.natan.cleanarchitecturecompanyservice.domain.exceptions.CompanyAlreadyExistsException;
import fr.natan.cleanarchitecturecompanyservice.domain.exceptions.CompanyAssociedProjectsException;
import fr.natan.cleanarchitecturecompanyservice.domain.exceptions.CompanyFieldsEmptyException;
import fr.natan.cleanarchitecturecompanyservice.domain.exceptions.CompanyNotFoundException;
import fr.natan.cleanarchitecturecompanyservice.domain.ports.input.CompanyInputSevice;
import fr.natan.cleanarchitecturecompanyservice.infrastructure.ports.input.feignclient.model.ProjectModel;
import fr.natan.cleanarchitecturecompanyservice.infrastructure.ports.output.models.CompanyDto;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
public class CompanyController {
    private final CompanyInputSevice companyInputSevice;

    public CompanyController(CompanyInputSevice companyInputSevice) {
        this.companyInputSevice = companyInputSevice;
    }

    @RequestMapping(value = "/companies", method = RequestMethod.GET)
    private List<Company> getAllCompanies()  {
        return companyInputSevice.getAllCompanies();
    }

    @RequestMapping(value = "/companies", method = RequestMethod.POST)
    private Company createCompany(@RequestBody CompanyDto companyDto) throws CompanyAlreadyExistsException, CompanyFieldsEmptyException {
        return companyInputSevice.createCompany(companyDto);
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.PUT)
    private Company updateCompany(@PathVariable(name = "companyID") String companyID, @RequestBody CompanyDto companyDto)
            throws CompanyNotFoundException, CompanyFieldsEmptyException, CompanyAlreadyExistsException {
        return companyInputSevice.updateCompany(companyID, companyDto);
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.GET)
    private Optional<Company> getCompanyByID(@PathVariable(name = "companyID") String companyID) throws CompanyNotFoundException {
        return companyInputSevice.getCompanyByID(companyID);
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.DELETE)
    private void deleteCompany(@PathVariable(name = "companyID") String companyID) throws CompanyNotFoundException, CompanyAssociedProjectsException {
        companyInputSevice.deleteCompany(companyID);
    }

    @RequestMapping(value = "/projects/companies/{companyID}", method = RequestMethod.GET)
    private List<ProjectModel> getProjectsAssignedoCompany(@PathVariable(name = "companyID") String companyID) throws CompanyNotFoundException {
        return companyInputSevice.getProjectsAssignedToCompany(companyID);
    }
}
