package fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.controller;

import fr.natan.cleanarchitectureemployeesservice.domain.entity.Employee;
import fr.natan.cleanarchitectureemployeesservice.domain.exception_metrier.*;
import fr.natan.cleanarchitectureemployeesservice.domain.ports.input.EmployeeInputService;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.models.ProjectModel;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.services.AddressServiceProxy;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.model.EmployeeDto;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@SuppressWarnings("CodeBlock2Expr")
@RestController
public class EmployeeController {
    public EmployeeInputService employeeInputService;
    private final AddressServiceProxy addressServiceProxy;

    public EmployeeController(EmployeeInputService employeeInputService, AddressServiceProxy addressServiceProxy) {
        this.employeeInputService = employeeInputService;
        this.addressServiceProxy = addressServiceProxy;
    }


    @RequestMapping(value = "/employees", method = RequestMethod.GET)
    private List<Employee> getEmployees(){
        List<Employee> employees = employeeInputService.getAllEmployees();
        employees.forEach(employee -> {
            employee.setAddress(addressServiceProxy.getAddressById(employee.getAddressID()));
        });

        return employees;
    }

    @RequestMapping(value = "/employees", method = RequestMethod.POST)
    private Employee createEmployee(@RequestBody EmployeeDto employeeDto) throws EmployeeFieldsInvalidException,
            EmployeeAlreadyExistsException, EmployeeCreationErrorDueToAddressAPIException {
        return employeeInputService.createEmployee(employeeDto);
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.PUT)
    private Employee updateEmployee(@PathVariable(name = "employeeID") String employeeID, @RequestBody EmployeeDto employeeDto)
            throws EmployeeNotFoundException, EmployeeFieldsInvalidException, EmployeeCreationErrorDueToAddressAPIException,
            EmployeeAlreadyExistsException {
        Employee employee = employeeInputService.updateEmployee(employeeID, employeeDto);
        employee.setAddress(addressServiceProxy.getAddressById(employee.getAddressID()));
        return employee;
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.GET)
    private Optional<Employee> getEmployeeByID(@PathVariable(name = "employeeID") String employeeID) throws EmployeeNotFoundException {
        Optional<Employee> employee = employeeInputService.getEmployeeByID(employeeID);
        employee.get().setAddress(addressServiceProxy.getAddressById(employee.get().getAddressID()));
        return employee;
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.DELETE)
    private void deleteEmployee(@PathVariable(name = "employeeID") String employeeID) throws EmployeeNotFoundException,
            EmployeeAssociatedProjectsException {
        employeeInputService.deleteEmployee(employeeID);
    }

    @RequestMapping(value = "/employees/addresses/{addressID}", method = RequestMethod.GET)
    private List<Employee> getEmployeesLivingGivenAddress(@PathVariable(name = "addressID") Long addressID){
        List<Employee> employees = employeeInputService.getEmployeesLivingAtGivenAddress(addressID);
        employees.forEach(employee -> {
            employee.setAddress(addressServiceProxy.getAddressById(employee.getAddressID()));
        });

        return employees;
    }
    @RequestMapping(value = "/projects/employees/{employeeID}", method = RequestMethod.GET)
    private List<ProjectModel> getProjectsAssignedToEmployee(@PathVariable(name = "employeeID") String employeeID) throws EmployeeNotFoundException {
        return employeeInputService.getProjectsAssignedToEmployee(employeeID);
    }
}
