package fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.service;

import fr.natan.cleanarchitectureemployeesservice.domain.entity.Employee;
import fr.natan.cleanarchitectureemployeesservice.domain.exception_metrier.EmployeeNotFoundException;
import fr.natan.cleanarchitectureemployeesservice.domain.ports.output.EmployeeOutputService;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.models.AddressModel;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.models.ProjectModel;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.services.AddressServiceProxy;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.input.feignclient.services.ProjectService;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.model.EmployeeDto;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.model.EmployeeModel;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.model_mapper.EmployeeMapper;
import fr.natan.cleanarchitectureemployeesservice.infrastructure.ports.output.repository.EmployeeRepository;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class EmployeeOutputServiceImpl implements EmployeeOutputService {
    private final EmployeeRepository employeeRepository;
    private final AddressServiceProxy addressServiceProxy;
    private final ProjectService projectService;
    public EmployeeOutputServiceImpl(EmployeeRepository employeeRepository, AddressServiceProxy addressServiceProxy,
                                     ProjectService projectService) {
        this.employeeRepository = employeeRepository;
        this.addressServiceProxy = addressServiceProxy;
        this.projectService = projectService;
    }
    @Override
    public List<Employee> getAllEmployees() {
        List<EmployeeModel> employeesModel = employeeRepository.findByOrderByHireDateDesc();
        return employeesModel
                .stream()
                .map(EmployeeMapper::mapEmployeeModelToClass)
                .collect(Collectors.toList());
    }
    @Override
    public Employee createEmployee(Employee employee){
        EmployeeModel employeeModel = EmployeeMapper.mapClassToModel(employee);
        EmployeeModel persistedEmployee = employeeRepository.save(employeeModel);

        return EmployeeMapper.mapEmployeeModelToClass(persistedEmployee);
    }
    @Override
    public AddressModel getAddressByID(Long addressID) {
        return addressServiceProxy.getAddressById(addressID);
    }
    @Override
    public List<Employee> getEmployeeByInfo(EmployeeDto employeeDto) {
        List<EmployeeModel> employeeModels = employeeRepository.findByFirstnameAndLastnameAndAddressID(
                employeeDto.getFirstname(), employeeDto.getLastname(), employeeDto.getAddressID());

        return employeeModels.stream()
                .map(EmployeeMapper::mapEmployeeModelToClass)
                .collect(Collectors.toList());
    }
    @Override
    public Optional<Employee> getEmployeeByID(String employeeID) throws EmployeeNotFoundException {
        EmployeeModel employeeModel = employeeRepository.findById(employeeID).orElseThrow(
                EmployeeNotFoundException::new
        );
        return Optional.of(EmployeeMapper.mapEmployeeModelToClass(employeeModel));
    }
    @Override
    public Employee updateEmployee(Employee employee) {
        EmployeeModel employeeModel = EmployeeMapper.mapClassToModel(employee);
        EmployeeModel persistedEmployee = employeeRepository.save(employeeModel);
        return EmployeeMapper.mapEmployeeModelToClass(persistedEmployee);
    }
    @Override
    public void deleteEmployee(Employee employee) {
        employeeRepository.delete(EmployeeMapper.mapClassToModel(employee));
    }

    @Override
    public List<Employee> getEmployeesLivingAtGivenAddress(Long addressID) {
        List<EmployeeModel> employeeModels = employeeRepository.findByAddressID(addressID);
        return employeeModels
                .stream()
                .map(EmployeeMapper::mapEmployeeModelToClass)
                .collect(Collectors.toList());
    }

    @Override
    public List<ProjectModel> getProjectsAssignedToEmployee(String employeeID) {
        return projectService.getProjectsAssignedToEmployee(employeeID);
    }
}
