package fr.natan.cleanarchitectureprojectservice.infranstructure.ports.input.controller;

import fr.natan.cleanarchitectureprojectservice.domain.entity.Project;
import fr.natan.cleanarchitectureprojectservice.domain.exceptions_metiers.*;
import fr.natan.cleanarchitectureprojectservice.domain.ports.input.ProjectInputService;
import fr.natan.cleanarchitectureprojectservice.infranstructure.ports.input.feignclients.entities.CompanyModel;
import fr.natan.cleanarchitectureprojectservice.infranstructure.ports.input.feignclients.entities.EmployeeModel;
import fr.natan.cleanarchitectureprojectservice.infranstructure.ports.input.feignclients.services.CompanyServiceProxy;
import fr.natan.cleanarchitectureprojectservice.infranstructure.ports.input.feignclients.services.EmployeeServiceProxy;
import fr.natan.cleanarchitectureprojectservice.infranstructure.ports.output.models.ProjectDto;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
public class ProjectController {
    private final ProjectInputService projectInputService;
    private final EmployeeServiceProxy employeeServiceProxy;
    private final CompanyServiceProxy companyServiceProxy;

    public ProjectController(ProjectInputService projectInputService,
                             EmployeeServiceProxy employeeServiceProxy,
                             CompanyServiceProxy companyServiceProxy) {
        this.projectInputService = projectInputService;
        this.employeeServiceProxy = employeeServiceProxy;
        this.companyServiceProxy = companyServiceProxy;
    }

    @RequestMapping(value = "/projects", method = RequestMethod.GET)
    private List<Project> getAllProjects() {
        List<Project> projects = projectInputService.getAllProjects();
        projects.forEach(project -> {
            project.setCompany(companyServiceProxy.getCompanyByID(project.getCompanyID()));
            project.setEmployee(employeeServiceProxy.getEmployeeByID(project.getEmployeeID()));
        });
        return projects;
    }

    @RequestMapping(value = "/projects", method = RequestMethod.POST)
    private Project createProject(@RequestBody ProjectDto projectDto) throws ProjectAlreadyExistsException,
            ProjectFieldsEmptyException, ProjectCreationErrorDueToEmployeeAPIException, ProjectCreationErrorDueToCompanyAPIException, ProjectCreationErrorDueToNotAcceptedEmployeeStateException {
        CompanyModel companyModel = companyServiceProxy.getCompanyByID(projectDto.getCompanyID());
        EmployeeModel employeeModel = employeeServiceProxy.getEmployeeByID(projectDto.getEmployeeID());
        Project createdProject = projectInputService.createProject(projectDto);
        createdProject.setEmployee(employeeServiceProxy.getEmployeeByID(createdProject.getEmployeeID()));
        createdProject.setCompany(companyServiceProxy.getCompanyByID(createdProject.getCompanyID()));
        return createdProject;
    }

    @RequestMapping(value = "/projects/{projectID}", method = RequestMethod.PUT)
    private Project updateProject(@PathVariable(name = "projectID") String projectID, @RequestBody ProjectDto projectDto) throws
            ProjectNotFoundException, ProjectCreationErrorDueToEmployeeAPIException, ProjectCreationErrorDueToCompanyAPIException,
            ProjectFieldsEmptyException, ProjectCreationErrorDueToNotAcceptedEmployeeStateException, ProjectAlreadyExistsException {
        Project updatedProject = projectInputService.updateProject(projectID, projectDto);
        updatedProject.setEmployee(employeeServiceProxy.getEmployeeByID(updatedProject.getEmployeeID()));
        updatedProject.setCompany(companyServiceProxy.getCompanyByID(updatedProject.getCompanyID()));
        return updatedProject;
    }

    @RequestMapping(value = "/projects/{projectID}")
    private void deleteProject(@PathVariable(name = "projectID") String projectID) throws ProjectNotFoundException,
            EmployeeIsAssiacetedToProjectException, CompanyIsAssiacetedToProjectException {
      projectInputService.deleteProject(projectID);
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.GET)
    private EmployeeModel getEmployee(@PathVariable(name = "employeeID") String employeeID) {
        return employeeServiceProxy.getEmployeeByID(employeeID);
    }

    private CompanyModel getCompany(@PathVariable(name = "companyID") String companyID) {
        return companyServiceProxy.getCompanyByID(companyID);
    }

    @RequestMapping(value = "/projects/{projectID}", method = RequestMethod.GET)
    private Optional<Project> getProject(@PathVariable(name = "projectID")String projectID) throws ProjectNotFoundException {
        Optional<Project> project = projectInputService.getProjectByID(projectID);
        project.get().setCompany(companyServiceProxy.getCompanyByID(project.get().getCompanyID()));
        project.get().setEmployee(employeeServiceProxy.getEmployeeByID(project.get().getEmployeeID()));

        return  project;
    }

    @RequestMapping(value = "/projects/companies/{companyID}", method = RequestMethod.GET)
    private List<Project> getProjectAssignedToCompany(@PathVariable(name = "companyID") String companyID) {
        List<Project> projects = projectInputService.getProjectsAssignedToCompany(companyID);
        projects.forEach(project -> {
            project.setCompany(companyServiceProxy.getCompanyByID(project.getCompanyID()));
            project.setEmployee(employeeServiceProxy.getEmployeeByID(project.getEmployeeID()));
        });

        return projects;
    }
@RequestMapping(value = "/projects/employees/{employeeID}", method = RequestMethod.GET)
    private List<Project> getProjectAssignedToEmployee(@PathVariable(name = "employeeID") String employeeID) {
        List<Project> projects = projectInputService.getProjectsAssignedToEmployee(employeeID);
        projects.forEach(project -> {
            project.setEmployee(employeeServiceProxy.getEmployeeByID(project.getEmployeeID()));
            project.setCompany(companyServiceProxy.getCompanyByID(project.getCompanyID()));
        });
        return projects;
    }
}
